@@ -39,6 +39,7 @@ from airflow.utils.session import create_session
 from airflow.utils.types import DagRunType
 
 from tests_common.test_utils.dag import sync_dag_to_db
+from tests_common.test_utils.taskinstance import create_task_instance
 from tests_common.test_utils.version_compat import AIRFLOW_V_3_0_PLUS
 
 DEFAULT_DATE = timezone.datetime(2016, 1, 1, 1, 0, 0)
@@ -81,7 +82,7 @@ def create_context(task, persist_to_db=False, map_index=None):
         from airflow.models.dag_version import DagVersion
 
         dag_version = DagVersion.get_latest_version(dag.dag_id)
-        task_instance = TaskInstance(task=task, run_id=dag_run.run_id, dag_version_id=dag_version.id)
+        task_instance = create_task_instance(task=task, run_id=dag_run.run_id, dag_version_id=dag_version.id)
     else:
         task_instance = TaskInstance(task=task, run_id=dag_run.run_id)
     task_instance.dag_run = dag_run
@@ -156,14 +157,14 @@ class TestKubernetesJobOperator:
         assert dag_id == rendered.container_resources.requests["cpu"]
         assert dag_id == rendered.volume_mounts[0].name
         assert dag_id == rendered.volume_mounts[0].sub_path
-        assert dag_id == ti.task.image
-        assert dag_id == ti.task.cmds
-        assert dag_id == ti.task.namespace
-        assert dag_id == ti.task.config_file
-        assert dag_id == ti.task.labels
-        assert dag_id == ti.task.job_template_file
-        assert dag_id == ti.task.arguments
-        assert dag_id == ti.task.env_vars[0]
+        assert dag_id == rendered.image
+        assert dag_id == rendered.cmds
+        assert dag_id == rendered.namespace
+        assert dag_id == rendered.config_file
+        assert dag_id == rendered.labels
+        assert dag_id == rendered.job_template_file
+        assert dag_id == rendered.arguments
+        assert dag_id == rendered.env_vars[0]
         assert dag_id == rendered.annotations["dag-id"]
 
     def sanitize_for_serialization(self, obj):
@@ -497,23 +498,54 @@ class TestKubernetesJobOperator:
         job = k.build_job_request_obj({})
         assert re.match(r"job-a-very-reasonable-task-name-[a-z0-9-]+", job.metadata.name) is not None
 
+    @pytest.mark.parametrize("randomize", [True, False])
     @pytest.mark.non_db_test_override
-    @patch(JOB_OPERATORS_PATH.format("KubernetesJobOperator.get_or_create_pod"))
     @patch(JOB_OPERATORS_PATH.format("KubernetesJobOperator.build_job_request_obj"))
     @patch(JOB_OPERATORS_PATH.format("KubernetesJobOperator.create_job"))
     @patch(HOOK_CLASS)
-    def test_execute(self, mock_hook, mock_create_job, mock_build_job_request_obj, mock_get_or_create_pod):
+    def test_name_normalized_on_execution(
+        self, mock_hook, mock_create_job, mock_build_job_request_obj, randomize
+    ):
+        """Test that names with underscores are normalized to hyphens on execution."""
+        name_base = "test_extra-123"
+        normalized_name = "test-extra-123"
+
+        mock_hook.return_value.is_job_failed.return_value = False
+        mock_job = mock.MagicMock()
+        mock_job.metadata.name = f"job-{normalized_name}"
+        mock_job.metadata.namespace = "default"
+        mock_create_job.return_value = mock_job
+        mock_ti = mock.MagicMock()
+        context = dict(ti=mock_ti)
+
+        op = KubernetesJobOperator(
+            name=name_base,
+            random_name_suffix=randomize,
+            task_id="task",
+        )
+
+        op.execute(context=context)
+
+        # Verify the name was normalized (underscore replaced with hyphen)
+        if randomize:
+            assert op.name.startswith(normalized_name)
+        else:
+            assert op.name == normalized_name
+
+    @pytest.mark.non_db_test_override
+    @patch(JOB_OPERATORS_PATH.format("KubernetesJobOperator.get_pods"))
+    @patch(JOB_OPERATORS_PATH.format("KubernetesJobOperator.build_job_request_obj"))
+    @patch(JOB_OPERATORS_PATH.format("KubernetesJobOperator.create_job"))
+    @patch(HOOK_CLASS)
+    def test_execute(self, mock_hook, mock_create_job, mock_build_job_request_obj, mock_get_pods):
         mock_hook.return_value.is_job_failed.return_value = False
         mock_job_request_obj = mock_build_job_request_obj.return_value
         mock_job_expected = mock_create_job.return_value
         mock_ti = mock.MagicMock()
         context = dict(ti=mock_ti)
 
-        op = KubernetesJobOperator(
-            task_id="test_task_id",
-        )
-        with pytest.warns(AirflowProviderDeprecationWarning):
-            execute_result = op.execute(context=context)
+        op = KubernetesJobOperator(task_id="test_task_id", wait_until_job_complete=False)
+        execute_result = op.execute(context=context)
 
         mock_build_job_request_obj.assert_called_once_with(context)
         mock_create_job.assert_called_once_with(job_request_obj=mock_job_request_obj)
@@ -537,20 +569,19 @@ class TestKubernetesJobOperator:
     @patch(JOB_OPERATORS_PATH.format("KubernetesJobOperator.create_job"))
     @patch(HOOK_CLASS)
     def test_execute_with_parallelism(
-        self, mock_hook, mock_create_job, mock_build_job_request_obj, mock_get_pods
+        self,
+        mock_hook,
+        mock_create_job,
+        mock_build_job_request_obj,
+        mock_get_pods,
     ):
         mock_hook.return_value.is_job_failed.return_value = False
         mock_job_request_obj = mock_build_job_request_obj.return_value
         mock_job_expected = mock_create_job.return_value
-        mock_get_pods.return_value = [mock.MagicMock(), mock.MagicMock()]
-        mock_pods_expected = mock_get_pods.return_value
         mock_ti = mock.MagicMock()
         context = dict(ti=mock_ti)
 
-        op = KubernetesJobOperator(
-            task_id="test_task_id",
-            parallelism=2,
-        )
+        op = KubernetesJobOperator(task_id="test_task_id", parallelism=2, wait_until_job_complete=False)
         execute_result = op.execute(context=context)
 
         mock_build_job_request_obj.assert_called_once_with(context)
@@ -565,15 +596,13 @@ class TestKubernetesJobOperator:
 
         assert op.job_request_obj == mock_job_request_obj
         assert op.job == mock_job_expected
-        assert op.pods == mock_pods_expected
-        with pytest.warns(AirflowProviderDeprecationWarning):
-            assert op.pod is mock_pods_expected[0]
+        mock_get_pods.assert_not_called()
         assert not op.wait_until_job_complete
         assert execute_result is None
         assert not mock_hook.wait_until_job_complete.called
 
     @pytest.mark.non_db_test_override
-    @patch(JOB_OPERATORS_PATH.format("KubernetesJobOperator.get_or_create_pod"))
+    @patch(JOB_OPERATORS_PATH.format("KubernetesJobOperator.get_pods"))
     @patch(JOB_OPERATORS_PATH.format("KubernetesJobOperator.build_job_request_obj"))
     @patch(JOB_OPERATORS_PATH.format("KubernetesJobOperator.create_job"))
     @patch(JOB_OPERATORS_PATH.format("KubernetesJobOperator.execute_deferrable"))
@@ -584,7 +613,7 @@ class TestKubernetesJobOperator:
         mock_execute_deferrable,
         mock_create_job,
         mock_build_job_request_obj,
-        mock_get_or_create_pod,
+        mock_get_pods,
     ):
         mock_hook.return_value.is_job_failed.return_value = False
         mock_job_request_obj = mock_build_job_request_obj.return_value
@@ -597,8 +626,7 @@ class TestKubernetesJobOperator:
             wait_until_job_complete=True,
             deferrable=True,
         )
-        with pytest.warns(AirflowProviderDeprecationWarning):
-            actual_result = op.execute(context=context)
+        actual_result = op.execute(context=context)
 
         mock_build_job_request_obj.assert_called_once_with(context)
         mock_create_job.assert_called_once_with(job_request_obj=mock_job_request_obj)
@@ -616,13 +644,11 @@ class TestKubernetesJobOperator:
         assert not mock_hook.wait_until_job_complete.called
 
     @pytest.mark.non_db_test_override
-    @patch(JOB_OPERATORS_PATH.format("KubernetesJobOperator.get_or_create_pod"))
+    @patch(JOB_OPERATORS_PATH.format("KubernetesJobOperator.get_pods"))
     @patch(JOB_OPERATORS_PATH.format("KubernetesJobOperator.build_job_request_obj"))
     @patch(JOB_OPERATORS_PATH.format("KubernetesJobOperator.create_job"))
     @patch(HOOK_CLASS)
-    def test_execute_fail(
-        self, mock_hook, mock_create_job, mock_build_job_request_obj, mock_get_or_create_pod
-    ):
+    def test_execute_fail(self, mock_hook, mock_create_job, mock_build_job_request_obj, mock_get_pods):
         mock_hook.return_value.is_job_failed.return_value = "Error"
 
         op = KubernetesJobOperator(
@@ -630,9 +656,8 @@ class TestKubernetesJobOperator:
             wait_until_job_complete=True,
         )
 
-        with pytest.warns(AirflowProviderDeprecationWarning):
-            with pytest.raises(AirflowException):
-                op.execute(context=dict(ti=mock.MagicMock()))
+        with pytest.raises(AirflowException):
+            op.execute(context=dict(ti=mock.MagicMock()))
 
     @pytest.mark.non_db_test_override
     @patch(JOB_OPERATORS_PATH.format("KubernetesJobOperator.defer"))
@@ -752,7 +777,7 @@ class TestKubernetesJobOperator:
         )
         assert actual_result is None
 
-    @patch(JOB_OPERATORS_PATH.format("KubernetesJobOperator.get_or_create_pod"))
+    @patch(JOB_OPERATORS_PATH.format("KubernetesJobOperator.get_pods"))
     @patch(JOB_OPERATORS_PATH.format("KubernetesJobOperator.build_job_request_obj"))
     @patch(JOB_OPERATORS_PATH.format("KubernetesJobOperator.create_job"))
     @patch(f"{HOOK_CLASS}.wait_until_job_complete")
@@ -761,16 +786,17 @@ class TestKubernetesJobOperator:
         mock_wait_until_job_complete,
         mock_create_job,
         mock_build_job_request_obj,
-        mock_get_or_create_pod,
+        mock_get_pods,
     ):
         mock_job_expected = mock_create_job.return_value
         mock_ti = mock.MagicMock()
+        mock_get_pods.return_value = [mock.MagicMock()]
+        mock_pods_expected = mock_get_pods.return_value
 
         op = KubernetesJobOperator(
             task_id="test_task_id", wait_until_job_complete=True, job_poll_interval=POLL_INTERVAL
         )
-        with pytest.warns(AirflowProviderDeprecationWarning):
-            op.execute(context=dict(ti=mock_ti))
+        op.execute(context=dict(ti=mock_ti))
 
         assert op.wait_until_job_complete
         assert op.job_poll_interval == POLL_INTERVAL
@@ -779,6 +805,9 @@ class TestKubernetesJobOperator:
             namespace=mock_job_expected.metadata.namespace,
             job_poll_interval=POLL_INTERVAL,
         )
+        assert op.pods == mock_pods_expected
+        with pytest.raises(AirflowProviderDeprecationWarning):
+            assert op.pod == mock_pods_expected[0]
 
     @pytest.mark.parametrize("do_xcom_push", [True, False])
     @pytest.mark.parametrize("get_logs", [True, False])
@@ -876,12 +905,11 @@ class TestKubernetesJobOperator:
         mock_client.delete_namespaced_job.assert_not_called()
         mock_serialize.assert_not_called()
 
-    @pytest.mark.parametrize("parallelism", [None, 2])
+    @pytest.mark.parametrize("parallelism", [1, 2])
     @pytest.mark.parametrize("do_xcom_push", [True, False])
     @pytest.mark.parametrize("get_logs", [True, False])
     @patch(JOB_OPERATORS_PATH.format("KubernetesJobOperator.extract_xcom"))
     @patch(JOB_OPERATORS_PATH.format("KubernetesJobOperator.get_pods"))
-    @patch(JOB_OPERATORS_PATH.format("KubernetesJobOperator.get_or_create_pod"))
     @patch(JOB_OPERATORS_PATH.format("KubernetesJobOperator.build_job_request_obj"))
     @patch(JOB_OPERATORS_PATH.format("KubernetesJobOperator.create_job"))
     @patch(f"{POD_MANAGER_CLASS}.fetch_requested_container_logs")
@@ -896,17 +924,18 @@ class TestKubernetesJobOperator:
         mocked_fetch_logs,
         mock_create_job,
         mock_build_job_request_obj,
-        mock_get_or_create_pod,
         mock_get_pods,
         mock_extract_xcom,
         get_logs,
         do_xcom_push,
         parallelism,
     ):
+        mock_pod_1 = mock.MagicMock()
         if parallelism == 2:
-            mock_pod_1 = mock.MagicMock()
             mock_pod_2 = mock.MagicMock()
             mock_get_pods.return_value = [mock_pod_1, mock_pod_2]
+        else:
+            mock_get_pods.return_value = [mock_pod_1]
         mock_ti = mock.MagicMock()
         op = KubernetesJobOperator(
             task_id="test_task_id",
@@ -917,11 +946,7 @@ class TestKubernetesJobOperator:
             parallelism=parallelism,
         )
 
-        if not parallelism:
-            with pytest.warns(AirflowProviderDeprecationWarning):
-                op.execute(context=dict(ti=mock_ti))
-        else:
-            op.execute(context=dict(ti=mock_ti))
+        op.execute(context=dict(ti=mock_ti))
 
         if do_xcom_push and not parallelism:
             mock_extract_xcom.assert_called_once()
@@ -937,6 +962,109 @@ class TestKubernetesJobOperator:
         else:
             mocked_fetch_logs.assert_not_called()
 
+    @pytest.mark.parametrize("retries", [3, 0])
+    @pytest.mark.parametrize("parallelism", [1, 2])
+    @patch(JOB_OPERATORS_PATH.format("KubernetesJobOperator.log_matching_pod"))
+    @patch(JOB_OPERATORS_PATH.format("KubernetesJobOperator._build_find_pod_label_selector"))
+    @patch(JOB_OPERATORS_PATH.format("KubernetesJobOperator.client"), new_callable=mock.PropertyMock)
+    def test_get_pods(
+        self, mock_client, mock_build_find_pod_label_selector, mock_log_matching_pod, parallelism, retries
+    ):
+        mock_context = mock.MagicMock()
+        mock_build_find_pod_label_selector.return_value = {
+            "dag_id": "fakedag",
+            "task_id": "faketask",
+            "run_id": "fakerun",
+            "kubernetes_pod_operator": "True",
+        }
+        mock_pod_1 = mock.MagicMock()
+        return_value = [mock_pod_1]
+        if parallelism == 2:
+            mock_pod_2 = mock.MagicMock()
+            return_value.append(mock_pod_2)
+        side_effects = []
+        for _i in range(retries):
+            side_effects.append(k8s.V1PodList(items=[]))
+        side_effects.append(k8s.V1PodList(items=return_value))
+        mock_client.return_value.list_namespaced_pod.side_effect = side_effects
+
+        op = KubernetesJobOperator(
+            task_id="faketask", parallelism=parallelism, discover_pods_retry_number=retries
+        )
+
+        result = op.get_pods(mock.MagicMock(), mock_context)
+
+        assert result == return_value
+
+        for pod in return_value:
+            mock_log_matching_pod.assert_any_call(pod=pod, context=mock_context)
+
+    @pytest.mark.parametrize("successful_try", [3, 1, 0])
+    @patch(JOB_OPERATORS_PATH.format("KubernetesJobOperator.log_matching_pod"))
+    @patch(JOB_OPERATORS_PATH.format("KubernetesJobOperator._build_find_pod_label_selector"))
+    @patch(JOB_OPERATORS_PATH.format("KubernetesJobOperator.client"), new_callable=mock.PropertyMock)
+    def test_get_pods_retry(
+        self, mock_client, mock_build_find_pod_label_selector, mock_log_matching_pod, successful_try
+    ):
+        retries = 3
+        mock_context = mock.MagicMock()
+        mock_build_find_pod_label_selector.return_value = {
+            "dag_id": "fakedag",
+            "task_id": "faketask",
+            "run_id": "fakerun",
+            "kubernetes_pod_operator": "True",
+        }
+
+        mock_pod_1 = mock.MagicMock()
+        return_value = [mock_pod_1]
+
+        side_effects = []
+        for i in range(retries + 1):
+            items = []
+            if i == successful_try:
+                items.append(mock_pod_1)
+            side_effects.append(k8s.V1PodList(items=items))
+        mock_client.return_value.list_namespaced_pod.side_effect = side_effects
+
+        op = KubernetesJobOperator(task_id="faketask", parallelism=1, discover_pods_retry_number=retries)
+
+        result = op.get_pods(mock.MagicMock(), mock_context)
+
+        assert result == return_value
+        assert mock_client.return_value.list_namespaced_pod.call_count == successful_try + 1
+
+    @patch(JOB_OPERATORS_PATH.format("KubernetesJobOperator.get_pods"))
+    @patch(JOB_OPERATORS_PATH.format("KubernetesJobOperator.build_job_request_obj"))
+    @patch(JOB_OPERATORS_PATH.format("KubernetesJobOperator.hook"), new_callable=mock.PropertyMock)
+    def test_create_zero_parallelism_job(self, mock_hook, mock_build_job_request_obj, mock_get_pods):
+        mock_context = mock.MagicMock()
+        mock_job = mock.MagicMock()
+        mock_job.to_dict.return_value = {}
+        mock_hook.return_value.get_namespace.return_value = "fakenamespace"
+        mock_build_job_request_obj.return_value = mock_job
+        op = KubernetesJobOperator(task_id="faketask", parallelism=0, wait_until_job_complete=False)
+
+        op.execute(mock_context)
+
+        mock_hook.return_value.create_job.assert_called_once_with(job=mock_job)
+        mock_get_pods.assert_not_called()
+
+    @patch(JOB_OPERATORS_PATH.format("KubernetesJobOperator.get_pods"))
+    @patch(JOB_OPERATORS_PATH.format("KubernetesJobOperator.build_job_request_obj"))
+    @patch(JOB_OPERATORS_PATH.format("KubernetesJobOperator.hook"), new_callable=mock.PropertyMock)
+    def test_create_zero_parallelism_fails_validation(
+        self, mock_hook, mock_build_job_request_obj, mock_get_pods
+    ):
+        mock_context = mock.MagicMock()
+        op = KubernetesJobOperator(task_id="faketask", parallelism=0, wait_until_job_complete=True)
+
+        with pytest.raises(AirflowException):
+            op.execute(mock_context)
+
+        mock_build_job_request_obj.assert_not_called()
+        mock_hook.return_value.create_job.assert_not_called()
+        mock_get_pods.assert_not_called()
+
 
 @pytest.mark.db_test
 @pytest.mark.execution_timeout(300)
